REBOL [
	Title: "REBOL/Services - Async TCP Client Handler"
	Author: "Carl Sassenrath"
	Notice: "Confidential source code. Under development."
	Copyright: "REBOL Technology 2005, All rights reserved."
]

;-- Asynchronous TCP Client Handler ------------------------------------------

;BUG: WAIT will wakeup even if the port is not specified in the
;wait list. If the port is not specified, it should not wakeup.
;Also: READ protocol error might blow it out to console.

add-client-handler [

	scheme: 'tcp

	opener: func [
		spec [port!]
		config [object!]
	][
		debug connect ['opening-tcp-port spec/host spec/port-id]
		; Create a session and open the port:
		make-session spec config
		if not spec/port-id [spec/port-id: rebol-port-id]
		open/direct/binary/async spec :handler
	]

	sender: func [
		port [port!]
		request
	][
		insert port mold-packet port/user-data request
	]

	set-wakeup: func [
		port [port!]
		flag [logic!]
	] [
		port/wakeup?: any [port/wakeup? flag]
	]

	clear-wakeup: func [
		port [port!]
	] [
		port/wakeup?: none
	]

	handler: func [port action arg /local session] [
		debug tcp ['handle-tcp action either object? arg ['object][arg]]
		session: port/user-data
		switch action [
			write-done [
				; Top item in queue is always the one just sent
				if arg: session/send-queue/1 [append session/sent-queue arg]
				remove session/send-queue
				send-request port
			]
			read [
				append session/recv-buf copy/part port arg
				debug tcp-read [arg 'bytes length? session/recv-buf 'total]
				while [ ; multiple requests may be concatenated in the buffer
					all [
						not empty? session/recv-buf
						arg: load-packet session
					]
				][wake-request port arg]
			]
			init [
				port/status: none
				set-modes port [timeout: session/port-timeout]
			]
			open [
				debug connect ['open port/remote-ip port/remote-port port/local-port]
				port/status: 'open
				send-request port
			]
			close [
				attempt [close port]
				port/status: none
			]
			error [print ["*** Client Error:" mold disarm :arg] close port]
		]
	]

]
