REBOL [
	Title: "REBOL/Services - Client Configuration"
	Author: "Carl Sassenrath"
	Notice: "Confidential source code. Under development."
	Copyright: "REBOL Technology 2005-2008, All rights reserved."
]

;-- Default Client Configuration ---------------------------------------------
;
;   Provides configuration for client connection. Many of these values
;   are also part of the server config and are used in functions that are
;   common to the client and the server. These values can be modified as
;   part of the /options block in Open-Service.
;
;   Within an initialized client, the config object is stored in the
;   session object (which can be found from port/user-data).

client-config: context [
	flags: []
	port-timeout: 20
	max-header:  2000		; Maximum header. Allows for trash at start.
	max-request: 100'000	; Maximum request size. Abort if greater.
	max-payload: 8'000'000	; Maximum payload size. Abort if greater.
	encode: pe-open			; packet encoding method (see pe values)
	open-key: default-key	; encryption key used for open access (CLOAK)
	priv-key: none			; encryption key used for private access (CLOAK, AES)
	rsa-key: none			; optional RSA key for certification (binary | object)
	callback: none			; request reply callback function
	log-file: %log.txt
	handler-context: none	; points to handler plugin
]

;-- Client Scheme Handler List
;
;   For each protocol scheme that you want to use, there must be a handler
;   object in this list. Examples: TCP, HTTP.

client-handlers: []

add-client-handler: func [
	spec [block!]
][
	spec: context spec
	repend client-handlers [spec/scheme spec] ; (error ok if no scheme field)
]

